@rem This file helps you using ANSI in your cmd scripts
@rem Just call this file and you can type in your prompt smth like
@rem
@rem   call SetEscChar
@rem   echo %ESC%[1;33;40m Yellow on black %ESC%[0m
@rem

@set ESC=
