@echo off

rem Run this file in cmd.exe or tcc.exe to change
rem color of command prompt to yellow on black.

rem Note!
rem "Inject ConEmuHk" and "ANSI X3.64" options
rem must be turned ON in ConEmu Settings!

rem prompt [1;33;40m$p$g[0;37;40m
prompt $e[1;33;40m$p$g$e[0;37;40m
